## Backtrace-SCA

- Backtrace service for ingesting and correlating Static Code Analysis reports

---

## Development
- `yarn install` or `npm install`

- Scripts 
  - `watch` rebuild application on file change and launch in development mode
  - `build` build the application in production mode
  - `start` start application in production mode (previous build required)
  - `start-dev` start application in development mode (previous build required)

- See `CONTRIBUTING.md` for guide on creating additional static code analysis tool integrations. 

---
#### Config

See `sca.conf` for configuration.

---

#### Submitting Defects

Use morgue to create an API token:
```
morgue login <url>
morgue token create --project=<project> --capability=sync:post --capability=error:post
morgue token list --project=<project>
```

Use curl to submit a file, including a `submitUrl` value.
You may optionally include a matchColumn value which should be the name of an attribute.
Any attributes that you would like submitted with the defect report should be appended to the endpoint as query string parameters. 
However, only the attribute with the matchColumn attribute name will be used specifically for correlation. This is useful for correlating defects to error objects based on build version.  

```
curl -X POST <sca-service-url>/submit/<sca-tool>?<some-attribute-name>=<some-attribute-value> \
  -H 'Content-Type: multipart/form-data' \
  -F report=@/path/to/report/report.tar.gz \
  -F projectName=<project-name> \
  -F 'submitUrl=https://<coroner-instance>:<submission-port>/post?format=json&token=<token>' \
  -F matchColumn=<some-attribute-name>
```

#### Correlation

Correlating crashes with SCA defects. PUT request triggers correlation processing, and GET request returns correlated defects for a given fingerprint.

```
curl -X PUT -H "Content-Type: application/json" "https://<sca-service-url>/correlate/<universe>" --data '{"fingerprint": <fingerprint>, .... }'
```

##### Correlation Parameters (all required)

* __crashProject__ //string// name of project where crash fingerprint originates 
* __defectProject__ //string// name of project to query for defects (defaults to the same project name the crash fingerprint comes from)
* __crashProjectToken__ //string// crash project token  
* __defectProjectToken__ //string// defect project token 
* __fingerprint__ //string// crash fingerprint
* __queryUrl__ //string// the URL of a coroner instance to query for defects and crashes

##### Correlation Options

* __classifierMatch__    consider a defect correlated to a crash only if it contains at least one classifier that the crash also contains
* __ignoreFiles__   //array of strings//  ignore files by name
* __ignoreFunctions__   //array of strings//  ignore functions by name
* __limitFiles__   //number//  only correlate against first n callstack files
* __limitFunctions__   //number//  only correlate against first n frames of callstack
* __matchColumn__   //string//  only match defects that share a value with crash object in this column. In particular, this is useful for matching the build version of the defects with that of the error object. The value you specify here should match an attribute key that you append as a query string parameter to the `/api/sca/submit/:format` endpoint when submitting defects. Please note that **this attribute must exist** in your coronerd instance or the correlation request will fail silently.

Example Data with Options:
```
{
    "fingerprint": "5860b8588ffae095ab8af305eab1d548e021148405aa54a90f43a398249721d1",
    "queryUrl": "https://my-coroner-instance.backtrace.io/api/query",
    "crashProject": "myProject", // project fingerprint comes from
    "crashProjectToken": "f00ba0d7254e3df5deec21980f0bce7aff2cfcf91b03e407288c5e107a0e5479e4",
    "defectProject": "MyProject-SCA",  // query project named "MyProject-SCA" for SCA defects
    "defectProjectToken": "f000d7254e3df5deec21980f0bce7aff2cfcf91b03e407288c5e107a0e5479e4",
    "classifierMatch": true,     // defects will have at least one matching classifier
    "ignoreFiles": ["main"],     // ignore files named 'main'
    "ignoreFunctions": ["foo"],  // ignore functions named 'foo'
    "limitFiles": 5,             // only correlate against first 5 callstack files
    "limitFunctions": 3,         // only correlate against first 3 frames of callstack
    "matchColumn": "build"       // only match defects with same "build" attribute value
}
```


GET Request:

Providing a query string parameter `build` with a string value of a build version returns only defects that were correlated with a crash of that build version. If no build param is supplied, defects correlated to the wildcard "*" build version are returned.

You can also specify query string parameter `fileName` or `functionName` to return defects that were correlated with that particular fileName or functionName of the crash callstack.

```
curl -X GET
    -H "X-Coroner-Token: <token>"
    -H "X-Coroner-Location: <location>"
    "https://<sca-service-url>/correlate/<universe>/<crash-project>/<crash-fingerprint>?build=<build-version>&fileName=<fileName>" 
```

```
curl -X GET
    -H "X-Coroner-Token: <token>"
    -H "X-Coroner-Location: <location>"
    "https://<sca-service-url>/correlate/myUniverse/myProject/9990b8588ffae095ab8af305eab1d548e021148405aa54a90f43a398249721d1?build='1.1.0'&fileName='test.c'" 
```

GET Response:
```
{
  "fingerprint": "9990b8588ffae095ab8af305eab1d548e021148405aa54a90f43a398249721d1",
  "universe": "myUniverse",
  "crashProject": "myProject",
  "build": "1.1.0",
  "functionName": "test.c",
  "defects": [
                { 
                    fingerprint: '5860b8588ffae095ab8af305eab1d548e021148405aa54a90f43a398249721d1',
                    errorMessage: '\'error->flags\' is used prior to initialization'
                }
            ]
}
```